#include "MPDF_UDF_FR.au3"

;Importe une image binaire du pressepapier et non le chemin d'une image
;E.g. Lancer le script, ouvrir Paint, faire un dessin, le slectionner et le copier => PDF
_SetTitle("Demo PDF in AutoIt")
_SetSubject("Demo PDF in AutoIt, without any ActiveX or DLL...")
_SetKeywords("pdf, demo, AutoIt")
_OpenAfter(True)
_SetUnit($PDF_UNIT_CM)
_SetPaperSize("A4")
_SetZoomMode($PDF_ZOOM_CUSTOM, 90)
_SetOrientation($PDF_ORIENTATION_PORTRAIT)
_SetLayoutMode($PDF_LAYOUT_CONTINOUS)
_InitPDF(@ScriptDir & "\test.pdf")
_LoadFontStandard("_Helv", $PDF_FONT_STD_HELVETICA, $PDF_FONT_BOLD)
_LoadMemoryImage("azerty")

_BeginPage()
_DrawText(3, 28, "Insertion d'une image du clipboard", "_Helv", 24, $PDF_ALIGN_LEFT)
_InsertImageMemory("azerty", 1, 28, 20)
_EndPage()
_ClosePDFFile()


; #FUNCTION# ====================================================================================================================
; Name ..........: _InsertImageMemory
; Description ...: Importe une image du presse papier
; Syntax ........: _InsertImage( $sAlias , $iX , $iY [, $iW ])
; Parameters ....: $sAlias              -  string value.
;                  $iX                  -  coordonne X du coin suprieur gauche.
;                  $iY                  -  coordonne Y du coin suprieur gauche.
;                  $iW                  - [optional]  largeur image destination.
; Return values .: None
; Author(s) .....: Mihai Iancu (taietel at yahoo dot com)
; Modified ......: Walkson
; Remarks .......: La hauteur est proportionnelle  la largeur de l'image de destination
; Related .......: _LoadMemoryImage()
; ===============================================================================================================================
Func _InsertImageMemory($sAlias, $iX, $iY, $iW = 0)
	If $iW = 0 Then
		$iW = $_iImageW/_GetUnit()
		$iH = $_iImageH/_GetUnit()
	Else
		$ratio = $_iImageH/$_iImageW;ratio image mmoire
		$iH = $iW * Round($ratio,4);applique ratio hauteur selon largeur image PDF
		$iY = $iY - $iH
	EndIf
	__ToBuffer("q" & @CRLF & __ToStr(__ToSpace($iW)) & " " & " 0 0 " & __ToStr(__ToSpace($iH)) & " " & __ToStr(__ToSpace($iX)) & " " & __ToStr(__ToSpace($iY)) & " cm" & _
	@CRLF & "/" & $sAlias & " Do" & @CRLF & "Q")
EndFunc   ;==>_InsertImage
; #FUNCTION# ====================================================================================================================
; Name ..........: _LoadMemoryImage
; Description ...: Charge une image dans la mmoire
; Syntax ........: _LoadResImage( $sImgAlias )
; Parameters ....: $sImgAlias           -  an alias to identify the image in the pdf (e.g. "Cheese").
; Return values .: Success      - True
;                  Failure      - False
; Author(s) .....: Mihai Iancu (taietel at yahoo dot com)
; Modified ......: Walkson
; Remarks .......: L'image doit tre en binaire et non le chemin d'une image e.g. copie de slection dans Paint
; Related .......: _ClipboardToStringImage()
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _LoadMemoryImage($sImgAlias )
	Local $iW, $iH, $ImgBuf, $hImage,$newImg, $iObj
	 _WaitClipboard()
	$imageV = _ClipboardToStringImage();clipboard format bmp
	$sImage = Binary($imageV)
	If $sImgAlias = "" Then __Error("You don't have an alias for the image", @ScriptLineNumber)
	If $sImage = ""  Then
		__Error("You don't have any images to insert or the path is invalid",@ScriptLineNumber)
	Else
				_GDIPlus_Startup()
				$hImage = _GDIPlus_BitmapCreateFromMemory($sImage)
				$bitmap = _Bitmap2BinaryString($hImage, 90);mise au format jpg

				$iW = _GDIPlus_ImageGetWidth($hImage)
				$iH = _GDIPlus_ImageGetHeight($hImage)

				ConsoleWrite($iW & " >> " & $iH & @CRLF)
				ConsoleWrite( _GetUnit() & @CRLF)

				$ImgBuf = BinaryToString($bitmap);mise en mmoire du jpg
				$_iImageW = $iW
				$_iImageH = $iH
				$iObj = __InitObj()
				__ToBuffer("<</Type /XObject /Subtype /Image /Name /" & $sImgAlias & " /Width " & $_iImageW & " /Height " & $_iImageH & " /Filter /DCTDecode /ColorSpace /DeviceRGB /BitsPerComponent 8" & " /Length " & $iObj + 1 & " 0 R" & ">>")
				__ToBuffer("stream" & @CRLF & $ImgBuf & @CRLF & "endstream")
				__EndObj()
				$_Image &= "/" & $sImgAlias & " " & $iObj & " 0 R " & @CRLF
				__InitObj()
				__ToBuffer(StringLen($ImgBuf))
				__EndObj()
				_GDIPlus_ImageDispose($hImage)
				_GDIPlus_Shutdown()

	EndIf
	Return $_Image
EndFunc   ;==>_LoadResImage
Func _WaitClipboard()
	If BinaryToString(  _ClipBoard_GetData($CF_DIB)) > "" Then $repond = MsgBox(4,"","Image en mmoire, voulez vous la rcuprer?")
	If BinaryToString( _ClipBoard_GetData($CF_DIB)) = ""  Then $repond = 7
	If $repond = 7 Then
		_ClipBoard_Open(0);vide le presse papier
		_ClipBoard_Empty()
		_ClipBoard_Close()
		ToolTip("En attente",100,50,"Presse papier")
		While 1;attente copie d'image
			Sleep(500)
			 $AAA = BinaryToString( _ClipBoard_GetData($CF_DIB))
			If  $AAA > "" Then ExitLoop
		WEnd
		ToolTip("")
	EndIf

EndFunc