
; #INDEX# =======================================================================================================================
; Title .........: Code length optimized Code128A/B/C barcode generator
; AutoIt Version : 3.3.12.0
; Description ...: Create a Code128A/B/C optimized barcode from supplied data
; Author(s) .....: David Williams (willichan)
; Dll ...........:
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
;_Bcode128_GenCode2pdf
; ===============================================================================================================================

;#include "StringSize.au3"; copi  la fin by Melba - http://www.autoitscript.com/forum/topic/114034-stringsize-m23-new-version-16-aug-11/
#include "..\MPDF_UDF_FR.au3"
; #EXEMPLE# =============================================================================================
_SetTitle("Demo PDF in AutoIt")
_SetSubject("Demo PDF in AutoIt, without any ActiveX or DLL...")
_SetKeywords("pdf, demo, AutoIt, Word")
_OpenAfter(True)
_SetUnit($PDF_UNIT_CM)
_SetPaperSize("A4")
_SetZoomMode($PDF_ZOOM_CUSTOM, 100)
_SetOrientation($PDF_ORIENTATION_PORTRAIT)
_SetLayoutMode($PDF_LAYOUT_CONTINOUS)
_InitPDF(@ScriptDir & "\test.pdf")
_Bcode128_GenCode2pdf("bonjour a tout le monde 642", "image")
_LoadFontTT("_Calibri", $PDF_FONT_CALIBRI)
_BeginPage()
_InsertRenderedText(10, 28, ">> PDF code barre 128 <<", "_Calibri", 32, 100, $PDF_ALIGN_CENTER, 0xFFFF00, 0xFF6600)
_InsertImage("image", 5, 25, 10, 1)
_EndPage()
_ClosePDFFile()
; #FUNCTION# ====================================================================================================================
; Name ..........: _Bcode128_GenCode2pdf
; Description ...: Creates a Code128A/B/C optimized barcode from supplied data to pdf
; Syntax ........: _Bcode128_GenCode2pdf($sData[, $sOutFile = Default[, $iDPI = Default[, $fMinBar = Default[, $fBarHeight = Default[,
;                  $iPrintText = Default]]]]])
; Parameters ....: $sData               - A string value to encode into a Code128 barcode
;                  $sImgAlias            - Alias
;                  $iDPI                - [optional] The dots-per-inch setting for the BMP file.  Default is 96
;                  $fMinBar             - [optional] Width of the narrowest bar, in inches (0.01 is the recommended standard for
;                                         most handheld scanners). Default is 0.01 inches
;                  $fBarHeight          - [optional] The height of the barcode, in inches (15% of the width or 0.5 inches
;                                         ,whichever is larger, is the standard according to the GS1-128 specification). Default
;                                         is to automatically calculate according to this standard.
;                  $iPrintText          - [optional] An integer value. Default is Default.
; Return values .: String containing the location and name of the BMP file created, or "" if copied to the clipboard.
; Author ........: David E Williams (willichan)
; Modified ......: Walkson
; Remarks .......: Invalid characters in $sData will be skipped rather than generating an error
; Related .......:
; Link ..........:http://www.autoitscript.com/forum/topic/165115-code128auto-barcode-generator/
; Example .......: No
; ===============================================================================================================================
Func _Bcode128_GenCode2pdf($sData, $sImgAlias, $iDPI = Default, $fMinBar = Default, $fBarHeight = Default, $iPrintText = Default)
	Local $vBarcodeObj[6], $t

	If $iPrintText = Default Then $iPrintText = True ;print text at bottom of barcode
	If $iDPI = Default Then $iDPI = 96 ;Dots Per Inch used for the image file
	If $fMinBar = Default Then $fMinBar = 0.01 ;Minimum bar width in Inches
	Local $iModWidth = Ceiling($iDPI * $fMinBar) ;Number of Dots per Minimum Bar
	$t = __Bcode128_EncodeData($sData)
	If $iPrintText Then
		$vBarcodeObj[0] = $t[1] ;encoded text
	Else
		$vBarcodeObj[0] = ""
	EndIf
	$vBarcodeObj[5] = $t[0] ;bar encoding
	$vBarcodeObj[1] = $iModWidth
	$vBarcodeObj[2] = $sImgAlias
	$vBarcodeObj[3] = $iDPI
	If $fBarHeight = Default Then
		$fBarHeight = ((StringLen($vBarcodeObj[5]) * $iModWidth) / $iDPI) * 0.15
		If $fBarHeight < 0.5 Then $fBarHeight = 0.5
	EndIf
	$fBarHeight = Ceiling($iDPI * $fBarHeight)
	$vBarcodeObj[4] = $fBarHeight
	Local $i, $cx
	Local Const $iFontMargin = 14
	;;create image
	_GDIPlus_Startup()
	Local $hBitmap = _GDIPlus_BitmapCreateFromScan0((StringLen($vBarcodeObj[5]) + 14) * $vBarcodeObj[1], $vBarcodeObj[4] + $iFontMargin, $GDIP_PXF01INDEXED)
	__Bcode128_BitmapSetResolution($hBitmap, $vBarcodeObj[3], $vBarcodeObj[3])
	Local $sCLSID = _GDIPlus_EncodersGetCLSID("bmp")
	Local $hBmpCtxt = _GDIPlus_ImageGetGraphicsContext($hBitmap)
	_GDIPlus_GraphicsSetSmoothingMode($hBmpCtxt, 0)
	_GDIPlus_GraphicsClear($hBmpCtxt, 0xFFFFFFFF)
	;;draw lines
	For $i = 1 To StringLen($vBarcodeObj[5])
		$cx = (($i - 1) + 7) * $vBarcodeObj[1]
		If StringMid($vBarcodeObj[5], $i, 1) = 1 Then
			_GDIPlus_GraphicsFillRect($hBmpCtxt, $cx, 0, $vBarcodeObj[1], $vBarcodeObj[4], 0)
			If @error Then ConsoleWrite("unable to draw bar " & $i & @CRLF)
		EndIf
	Next

	If $vBarcodeObj[0] <> "" Then
		$i = _StringSize($vBarcodeObj[0], 8, Default, Default, "Arial")
		$cx = ((StringLen($vBarcodeObj[5]) * $vBarcodeObj[1]) / 2) - ($i[2] / 2)
		_GDIPlus_GraphicsDrawString($hBmpCtxt, $vBarcodeObj[0], $cx, $vBarcodeObj[4] + 1, "Arial", 8)
		If @error Then ConsoleWrite("unable to draw text - " & @extended & @CRLF)
	EndIf
				$bitmap = _Bitmap2BinaryString($hBitmap, 90);mise au format jpg
				$iW = _GDIPlus_ImageGetWidth($hBitmap)
				$iH = _GDIPlus_ImageGetHeight($hBitmap)

				ConsoleWrite($iW & " >> " & $iH & @CRLF)
				$ImgBuf = BinaryToString($bitmap);mise en mmoire du jpg
				$_iImageW = $iW
				$_iImageH = $iH
				$iObj = __InitObj()
				__ToBuffer("<</Type /XObject /Subtype /Image /Name /" & $sImgAlias & " /Width " & $_iImageW & " /Height " & $_iImageH & " /Filter /DCTDecode /ColorSpace /DeviceRGB /BitsPerComponent 8" & " /Length " & $iObj + 1 & " 0 R" & ">>")
				__ToBuffer("stream" & @CRLF & $ImgBuf & @CRLF & "endstream")
				__EndObj()
				$_Image &= "/" & $sImgAlias & " " & $iObj & " 0 R " & @CRLF
				__InitObj()
				__ToBuffer(StringLen($ImgBuf))
				__EndObj()
	$bitmap = 0
	_GDIPlus_GraphicsDispose($hBmpCtxt)
	_GDIPlus_BitmapDispose($hBitmap)
	_GDIPlus_Shutdown()
EndFunc   ;==>_Bcode128_GenCode2pdf

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __Bcode128_AddChecksum
; Description ...: Updated the checksum for the current code
; Syntax ........: __Bcode128_AddChecksum(Byref $iChecksum, Byref $iC, $sChar, $sType, Byref $aEncoding)
; Parameters ....: $iChecksum           - current checksum
;                  $iC                  - checksum position counter
;                  $sChar               - character to add to the checksum
;                  $sType               - current codeset
;                  $aEncoding           - array of codeset data
; Return values .: None
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __Bcode128_AddChecksum(ByRef $iChecksum, ByRef $iC, $sChar, $sType, ByRef $aEncoding)
	$iC += 1
	$iChecksum += (__Bcode128_GetValue($sChar, $sType, $aEncoding) * $iC)
EndFunc   ;==>__Bcode128_AddChecksum

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __Bcode128_BitmapSetResolution
; Description ...: Sets the resolution of this Bitmap object
; Syntax ........: __Bcode128_BitmapSetResolution($hBitmap, $nDpiX, $nDpiY)
; Parameters ....: $hBitmap - Pointer to the Bitmap object
;                  $nDpiX   - Value that specifies the horizontal resolution in dots per inch.
;                  $nDpiX   - Value that specifies the vertical resolution in dots per inch.
; Return values .: Success  - True
;                  Failure  - False, @error and @extended are set if DllCall failed
; Author ........: UEZ
; Modified ......: funkey, willichan
; Remarks .......:
; Related .......:
; Link ..........: http://www.autoitscript.com/forum/topic/164388-changing-the-dpi-of-a-bmp
; Example .......: No
; ===============================================================================================================================
Func __Bcode128_BitmapSetResolution($hBitmap, $nDpiX, $nDpiY)
	Local $aResult = DllCall($__g_hGDIPDll, "uint", "GdipBitmapSetResolution", "handle", $hBitmap, "float", $nDpiX, "float", $nDpiY)
	If @error Then Return SetError(@error, @extended, False)
	Return $aResult[0] = 0
EndFunc   ;==>__Bcode128_BitmapSetResolution

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __Bcode128_DataInit
; Description ...: loads array with coding data
; Syntax ........: __Bcode128_DataInit(Byref $aEncoding)
; Parameters ....: $aEncoding           - variable to fill with coding data array
; Return values .: None
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __Bcode128_DataInit(ByRef $aEncoding)
	;;;Value, Acode, Bcode, Ccode, Bar Pattern
	Local $aEnc[107][5] = [ _
		["0", " ", " ", "00", "11011001100"], _
		["1", "!", "!", "01", "11001101100"], _
		["2", '"', '"', "02", "11001100110"], _
		["3", "#", "#", "03", "10010011000"], _
		["4", "$", "$", "04", "10010001100"], _
		["5", "%", "%", "05", "10001001100"], _
		["6", "&", "&", "06", "10011001000"], _
		["7", "'", "'", "07", "10011000100"], _
		["8", "(", "(", "08", "10001100100"], _
		["9", ")", ")", "09", "11001001000"], _
		["10", "*", "*", "10", "11001000100"], _
		["11", "+", "+", "11", "11000100100"], _
		["12", ",", ",", "12", "10110011100"], _
		["13", "-", "-", "13", "10011011100"], _
		["14", ".", ".", "14", "10011001110"], _
		["15", "/", "/", "15", "10111001100"], _
		["16", "0", "0", "16", "10011101100"], _
		["17", "1", "1", "17", "10011100110"], _
		["18", "2", "2", "18", "11001110010"], _
		["19", "3", "3", "19", "11001011100"], _
		["20", "4", "4", "20", "11001001110"], _
		["21", "5", "5", "21", "11011100100"], _
		["22", "6", "6", "22", "11001110100"], _
		["23", "7", "7", "23", "11101101110"], _
		["24", "8", "8", "24", "11101001100"], _
		["25", "9", "9", "25", "11100101100"], _
		["26", ":", ":", "26", "11100100110"], _
		["27", ";", ";", "27", "11101100100"], _
		["28", "<", "<", "28", "11100110100"], _
		["29", "=", "=", "29", "11100110010"], _
		["30", ">", ">", "30", "11011011000"], _
		["31", "?", "?", "31", "11011000110"], _
		["32", "@", "@", "32", "11000110110"], _
		["33", "A", "A", "33", "10100011000"], _
		["34", "B", "B", "34", "10001011000"], _
		["35", "C", "C", "35", "10001000110"], _
		["36", "D", "D", "36", "10110001000"], _
		["37", "E", "E", "37", "10001101000"], _
		["38", "F", "F", "38", "10001100010"], _
		["39", "G", "G", "39", "11010001000"], _
		["40", "H", "H", "40", "11000101000"], _
		["41", "I", "I", "41", "11000100010"], _
		["42", "J", "J", "42", "10110111000"], _
		["43", "K", "K", "43", "10110001110"], _
		["44", "L", "L", "44", "10001101110"], _
		["45", "M", "M", "45", "10111011000"], _
		["46", "N", "N", "46", "10111000110"], _
		["47", "O", "O", "47", "10001110110"], _
		["48", "P", "P", "48", "11101110110"], _
		["49", "Q", "Q", "49", "11010001110"], _
		["50", "R", "R", "50", "11000101110"], _
		["51", "S", "S", "51", "11011101000"], _
		["52", "T", "T", "52", "11011100010"], _
		["53", "U", "U", "53", "11011101110"], _
		["54", "V", "V", "54", "11101011000"], _
		["55", "W", "W", "55", "11101000110"], _
		["56", "X", "X", "56", "11100010110"], _
		["57", "Y", "Y", "57", "11101101000"], _
		["58", "Z", "Z", "58", "11101100010"], _
		["59", "[", "[", "59", "11100011010"], _
		["60", "\", "\", "60", "11101111010"], _
		["61", "]", "]", "61", "11001000010"], _
		["62", "^", "^", "62", "11110001010"], _
		["63", "_", "_", "63", "10100110000"], _
		["64", Chr(0), "`", "64", "10100001100"], _
		["65", Chr(1), "a", "65", "10010110000"], _
		["66", Chr(2), "b", "66", "10010000110"], _
		["67", Chr(3), "c", "67", "10000101100"], _
		["68", Chr(4), "d", "68", "10000100110"], _
		["69", Chr(5), "e", "69", "10110010000"], _
		["70", Chr(6), "f", "70", "10110000100"], _
		["71", Chr(7), "g", "71", "10011010000"], _
		["72", Chr(8), "h", "72", "10011000010"], _
		["73", Chr(9), "i", "73", "10000110100"], _
		["74", Chr(10), "j", "74", "10000110010"], _
		["75", Chr(11), "k", "75", "11000010010"], _
		["76", Chr(12), "l", "76", "11001010000"], _
		["77", Chr(13), "m", "77", "11110111010"], _
		["78", Chr(14), "n", "78", "11000010100"], _
		["79", Chr(15), "o", "79", "10001111010"], _
		["80", Chr(16), "p", "80", "10100111100"], _
		["81", Chr(17), "q", "81", "10010111100"], _
		["82", Chr(18), "r", "82", "10010011110"], _
		["83", Chr(19), "s", "83", "10111100100"], _
		["84", Chr(20), "t", "84", "10011110100"], _
		["85", Chr(21), "u", "85", "10011110010"], _
		["86", Chr(22), "v", "86", "11110100100"], _
		["87", Chr(23), "w", "87", "11110010100"], _
		["88", Chr(24), "x", "88", "11110010010"], _
		["89", Chr(25), "y", "89", "11011011110"], _
		["90", Chr(26), "z", "90", "11011110110"], _
		["91", Chr(27), "{", "91", "11110110110"], _
		["92", Chr(28), "|", "92", "10101111000"], _
		["93", Chr(29), "}", "93", "10100011110"], _
		["94", Chr(30), "~", "94", "10001011110"], _
		["95", Chr(31), Chr(127), "95", "10111101000"], _
		["96", "Fnc3", "Fnc3", "96", "10111100010"], _
		["97", "Fnc2", "Fnc2", "97", "11110101000"], _
		["98", "Shift B", "Shift A", "98", "11110100010"], _
		["99", "Code C", "Code C", "99", "10111011110"], _
		["100", "Code B", "Fnc 4", "Code B", "10111101110"], _
		["101", "Fnc 4", "Code A", "Code A", "11101011110"], _
		["102", "Fnc 1", "Fnc 1", "Fnc 1", "11110101110"], _
		["103", "Start A", "Start A", "Start A", "11010000100"], _
		["104", "Start B", "Start B", "Start B", "11010010000"], _
		["105", "Start C", "Start C", "Start C", "11010011100"], _
		["106", "Stop", "Stop", "Stop", "1100011101011"] _
		]
	$aEncoding = $aEnc
	$aEnc = 0
EndFunc

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __Bcode128_EncodeData
; Description ...: encodes data into bar/space code for generating barcode
; Syntax ........: __Bcode128_EncodeData($sData)
; Parameters ....: $sData               - data to be converted to a barcode
; Return values .: array contining encoded data
;                      [0] - text for placing at the bottom of the barcode
;                      [1] - bar/space coding to generate barcode
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __Bcode128_EncodeData($sData)
	Local $sType = "" ;Start with no type
	Local $sNextType = ""
	Local $iChecksum = 0
	Local $aReturn[2] = ["", ""]
	Local $iPosition = 0
	Local $t, $iC = 0
	Local $aEncoding
	__Bcode128_DataInit($aEncoding)
	While $iPosition < StringLen($sData)
		$iPosition += 1
		$sNextType = __Bcode128_GetNextType($sData, $iPosition, $sType)
		If $sNextType <> "" Then
			If $sNextType <> $sType Then ;Start code or switch code
				If $aReturn[0] = "" Then $sNextType &= "S"
				Switch $sNextType
					Case "A" ;Change to Code128A
						$aReturn[0] &= __Bcode128_GetBars("Code A", $sType, $aEncoding)
						__Bcode128_AddChecksum($iChecksum, $iC, "Code A", $sType, $aEncoding)
					Case "AS" ;Start Code128A
						$iChecksum = __Bcode128_GetValue("Start A", "A", $aEncoding)
						$aReturn[0] = __Bcode128_GetBars("Start A", "A", $aEncoding)
					Case "B" ;Change to Code128B
						$aReturn[0] &= __Bcode128_GetBars("Code B", $sType, $aEncoding)
						__Bcode128_AddChecksum($iChecksum, $iC, "Code B", $sType, $aEncoding)
					Case "BS" ;Start Code128B
						$iChecksum = __Bcode128_GetValue("Start B", "B", $aEncoding)
						$aReturn[0] = __Bcode128_GetBars("Start B", "B", $aEncoding)
					Case "C" ;Change to Code128C
						$aReturn[0] &= __Bcode128_GetBars("Code C", $sType, $aEncoding)
						__Bcode128_AddChecksum($iChecksum, $iC, "Code C", $sType, $aEncoding)
					Case "CS" ;Start Code128C
						$iChecksum = __Bcode128_GetValue("Start C", "C", $aEncoding)
						$aReturn[0] = __Bcode128_GetBars("Start C", "C", $aEncoding)
				EndSwitch
				$sType = StringLeft($sNextType, 1)
			EndIf
			If $sType = "C" Then
				$t = StringMid($sData, $iPosition, 2)
				$iPosition += 1
			Else
				$t = StringMid($sData, $iPosition, 1)
			EndIf
			$aReturn[0] &= __Bcode128_GetBars($t, $sType, $aEncoding)
			$aReturn[1] &= __Bcode128_IsPrintable($t)
			__Bcode128_AddChecksum($iChecksum, $iC, $t, $sType, $aEncoding)
		EndIf
	WEnd
	$iChecksum = Mod($iChecksum, 103)
	$aReturn[0] &= __Bcode128_GetBarsFromVal($iChecksum, $aEncoding)
	$aReturn[0] &= __Bcode128_GetBarsFromVal(106, $aEncoding)
	Return $aReturn
EndFunc   ;==>__Bcode128_EncodeData

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __Bcode128_GetBars
; Description ...: returns the bar/space coding for a character
; Syntax ........: __Bcode128_GetBars($sChar, $sType, Byref $aEncoding)
; Parameters ....: $sChar               - Character to lookup
;                  $sType               - current characterset
;                  $aEncoding           - array of encoding data
; Return values .: string containing bar/space encoding
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __Bcode128_GetBars($sChar, $sType, ByRef $aEncoding)
	Local $i, $iType
	Switch StringUpper($sType)
		Case "A"
			$iType = 1
		Case "B"
			$iType = 2
		Case "C"
			$iType = 3
		Case Else
			SetError(1, 0, -1)
	EndSwitch
	For $i = 0 to UBound($aEncoding, 1) - 1
		If $aEncoding[$i][$iType] == $sChar Then Return $aEncoding[$i][4]
	Next
	SetError(1, 0, "")
EndFunc

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __Bcode128_GetBarsFromVal
; Description ...: returns the bar/space coding for a codeset value
; Syntax ........: __Bcode128_GetBarsFromVal($iValue, Byref $aEncoding)
; Parameters ....: $iValue              - codeset value to lookup
;                  $aEncoding           - array of encoding data
; Return values .: string containing bar/space encoding
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __Bcode128_GetBarsFromVal($iValue, ByRef $aEncoding)
	Local $i
	For $i = 0 to UBound($aEncoding, 1) - 1
		If $aEncoding[$i][0] = $iValue Then Return $aEncoding[$i][4]
	Next
	SetError(1, 0, -1)
EndFunc

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __Bcode128_GetNextType
; Description ...: determines the next codeset to use for an optimized barcode
; Syntax ........: __Bcode128_GetNextType($sData, $iPosition, $sType)
; Parameters ....: $sData               - the data being encoded
;                  $iPosition           - current position being encoded
;                  $sType               - current codeset
; Return values .: string containing the optimum codeset to be used for the next character
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __Bcode128_GetNextType($sData, $iPosition, $sType)
	;Optimizes the generated code lenth
	Local $sSet
	Local $iA = 0
	Local $iB = 0
	Local $iP = 0
	;Check to see if it should be "C"
	If $sType = "C" Then
		If StringRegExp(StringMid($sData, $iPosition, 2), "([0-9][0-9])") = 1 Then Return "C"
	Else
		If StringRegExp(StringMid($sData, $iPosition, 6), "([0-9][0-9]){3}") = 1 Then Return "C"
	EndIf
	;Check "A" or "B"
	$sSet = __Bcode128_WhichSet(StringMid($sData, $iPosition, 1))
	If StringLen($sSet) < 2 Then Return $sSet
	If ($sType = "A") And StringInStr($sSet, "A") Then Return "A"
	If ($sType = "B") And StringInStr($sSet, "B") Then Return "B"
	While 1
		$iP += 1
		If $iPosition + $iP > StringLen($sData) Then ExitLoop
		$sSet = __Bcode128_WhichSet(StringMid($sData, $iPosition + $iP, 1))
		If StringInStr($sSet, "A") Then $iA += 1
		If StringInStr($sSet, "B") Then $iB += 1
		If StringLen($sSet) = 1 Then ExitLoop
	WEnd
	If $iA > $iB Then Return "A"
	Return "B"
EndFunc   ;==>__Bcode128_GetNextType

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __Bcode128_GetValue
; Description ...: returns the codeset value for a character
; Syntax ........: __Bcode128_GetValue($sChar, $sType, Byref $aEncoding)
; Parameters ....: $sChar               - character to lookup
;                  $sType               - current codeset
;                  $aEncoding           - array of encoding data
; Return values .: codeset value of the character
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __Bcode128_GetValue($sChar, $sType, ByRef $aEncoding)
	Local $i, $iType
	Switch StringUpper($sType)
		Case "A"
			$iType = 1
		Case "B"
			$iType = 2
		Case "C"
			$iType = 3
		Case Else
			SetError(1, 0, -1)
	EndSwitch
	For $i = 0 to UBound($aEncoding, 1) - 1
		If $aEncoding[$i][$iType] == $sChar Then Return $aEncoding[$i][0]
	Next
	SetError(1, 0, $aEncoding[0][4])
EndFunc

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __Bcode128_IsPrintable
; Description ...:
; Syntax ........: __Bcode128_IsPrintable($sChar)
; Parameters ....: $sChar               - character to evaluate
; Return values .: printable characters associated with the given character
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __Bcode128_IsPrintable($sChar)
	;eventually add printable forms of non-printable characters, such as <CR>
	If StringLen($sChar) = 2 Then Return $sChar ;it is a double number
	Switch Asc($sChar)
		Case 32 To 126
			Return $sChar
		Case Else
			Return ""
	EndSwitch
EndFunc   ;==>__Bcode128_IsPrintable

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __Bcode128_WhichSet
; Description ...: Determines which charactersets the character is valid in (A and B only)
; Syntax ........: __Bcode128_WhichSet($sChar)
; Parameters ....: $sChar               - Character to lookup
; Return values .: string containing the valid codesets
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __Bcode128_WhichSet($sChar)
	If (Not IsString($sChar)) Or (StringLen($sChar) > 1) Then SetError(1, 0, -1)
	Switch Asc($sChar)
		Case 0 To 31
			Return "A"
		Case 32 To 95
			Return "AB"
		Case 96 To 127
			Return "B"
		Case Else
			Return ""
	EndSwitch
EndFunc



;############################################################################################################

; #INDEX# ============================================================================================================
; Title .........: _StringSize
; AutoIt Version : v3.2.12.1 or higher
; Language ......: English
; Description ...: Returns size of rectangle required to display string - maximum width can be chosen
; Remarks .......:
; Note ..........:
; Author(s) .....:  Melba23 - thanks to trancexx for the default DC code
; ====================================================================================================================

;#AutoIt3Wrapper_Au3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6

; #CURRENT# ==========================================================================================================
; _StringSize: Returns size of rectangle required to display string - maximum width can be chosen
; ====================================================================================================================

; #INTERNAL_USE_ONLY#=================================================================================================
; _StringSize_Error_Close: Releases DC and deletes font object after error
; _StringSize_DefaultFontName: Determines Windows default font
; ====================================================================================================================

; #FUNCTION# =========================================================================================================
; Name...........: _StringSize
; Description ...: Returns size of rectangle required to display string - maximum permitted width can be chosen
; Syntax ........: _StringSize($sText[, $iSize[, $iWeight[, $iAttrib[, $sName[, $iWidth[, $hWnd]]]]]])
; Parameters ....: $sText   - String to display
;                  $iSize   - [optional] Font size in points - (default = 8.5)
;                  $iWeight - [optional] Font weight - (default = 400 = normal)
;                  $iAttrib - [optional] Font attribute (0-Normal (default), 2-Italic, 4-Underline, 8 Strike)
;                             + 1 if tabs are to be expanded before sizing
;                  $sName   - [optional] Font name - (default = Tahoma)
;                  $iWidth  - [optional] Max width for rectangle - (default = 0 => width of original string)
;                  $hWnd    - [optional] GUI in which string will be displayed - (default 0 => normally not required)
; Requirement(s) : v3.2.12.1 or higher
; Return values .: Success - Returns 4-element array: ($iWidth set // $iWidth not set)
;                  |$array[0] = String reformatted with additonal @CRLF // Original string
;                  |$array[1] = Height of single line in selected font // idem
;                  |$array[2] = Width of rectangle required for reformatted // original string
;                  |$array[3] = Height of rectangle required for reformatted // original string
;                  Failure - Returns 0 and sets @error:
;                  |1 - Incorrect parameter type (@extended = parameter index)
;                  |2 - DLL call error - extended set as follows:
;                       |1 - GetDC failure
;                       |2 - SendMessage failure
;                       |3 - GetDeviceCaps failure
;                       |4 - CreateFont failure
;                       |5 - SelectObject failure
;                       |6 - GetTextExtentPoint32 failure
;                  |3 - Font too large for chosen max width - a word will not fit
; Author ........: Melba23 - thanks to trancexx for the default DC code
; Modified ......:
; Remarks .......: The use of the $hWnd parameter is not normally necessary - it is only required if the UDF does not
;                   return correct dimensions without it.
; Related .......:
; Link ..........:
; Example .......: Yes
;=====================================================================================================================
Func _StringSize($sText, $iSize = 8.5, $iWeight = 400, $iAttrib = 0, $sName = "", $iMaxWidth = 0, $hWnd = 0)

	; Set parameters passed as Default
	If $iSize = Default Then $iSize = 8.5
	If $iWeight = Default Then $iWeight = 400
	If $iAttrib = Default Then $iAttrib = 0
	If $sName = "" Or $sName = Default Then $sName = _StringSize_DefaultFontName()

	; Check parameters are correct type
	If Not IsString($sText) Then Return SetError(1, 1, 0)
	If Not IsNumber($iSize) Then Return SetError(1, 2, 0)
	If Not IsInt($iWeight) Then Return SetError(1, 3, 0)
	If Not IsInt($iAttrib) Then Return SetError(1, 4, 0)
	If Not IsString($sName) Then Return SetError(1, 5, 0)
	If Not IsNumber($iMaxWidth) Then Return SetError(1, 6, 0)
	If Not IsHWnd($hWnd) And $hWnd <> 0 Then Return SetError(1, 7, 0)

	Local $aRet, $hDC, $hFont, $hLabel = 0, $hLabel_Handle

	; Check for tab expansion flag
	Local $iExpTab = BitAND($iAttrib, 1)
	; Remove possible tab expansion flag from font attribute value
	$iAttrib = BitAND($iAttrib, BitNOT(1))

	; If GUI handle was passed
	If IsHWnd($hWnd) Then
		; Create label outside GUI borders
		$hLabel = GUICtrlCreateLabel("", -10, -10, 10, 10)
		$hLabel_Handle = GUICtrlGetHandle(-1)
		GUICtrlSetFont(-1, $iSize, $iWeight, $iAttrib, $sName)
		; Create DC
		$aRet = DllCall("user32.dll", "handle", "GetDC", "hwnd", $hLabel_Handle)
		If @error Or $aRet[0] = 0 Then
			GUICtrlDelete($hLabel)
			Return SetError(2, 1, 0)
		EndIf
		$hDC = $aRet[0]
		$aRet = DllCall("user32.dll", "lparam", "SendMessage", "hwnd", $hLabel_Handle, "int", 0x0031, "wparam", 0, "lparam", 0) ; $WM_GetFont
		If @error Or $aRet[0] = 0 Then
			GUICtrlDelete($hLabel)
			Return SetError(2, _StringSize_Error_Close(2, $hDC), 0)
		EndIf
		$hFont = $aRet[0]
	Else
		; Get default DC
		$aRet = DllCall("user32.dll", "handle", "GetDC", "hwnd", $hWnd)
		If @error Or $aRet[0] = 0 Then Return SetError(2, 1, 0)
		$hDC = $aRet[0]
		; Create required font
		$aRet = DllCall("gdi32.dll", "int", "GetDeviceCaps", "handle", $hDC, "int", 90) ; $LOGPIXELSY
		If @error Or $aRet[0] = 0 Then Return SetError(2, _StringSize_Error_Close(3, $hDC), 0)
		Local $iInfo = $aRet[0]
		$aRet = DllCall("gdi32.dll", "handle", "CreateFontW", "int", -$iInfo * $iSize / 72, "int", 0, "int", 0, "int", 0, _
				"int", $iWeight, "dword", BitAND($iAttrib, 2), "dword", BitAND($iAttrib, 4), "dword", BitAND($iAttrib, 8), "dword", 0, "dword", 0, _
				"dword", 0, "dword", 5, "dword", 0, "wstr", $sName)
		If @error Or $aRet[0] = 0 Then Return SetError(2, _StringSize_Error_Close(4, $hDC), 0)
		$hFont = $aRet[0]
	EndIf

	; Select font and store previous font
	$aRet = DllCall("gdi32.dll", "handle", "SelectObject", "handle", $hDC, "handle", $hFont)
	If @error Or $aRet[0] = 0 Then Return SetError(2, _StringSize_Error_Close(5, $hDC, $hFont, $hLabel), 0)
	Local $hPrevFont = $aRet[0]

	; Declare variables
	Local $avSize_Info[4], $iLine_Length, $iLine_Height = 0, $iLine_Count = 0, $iLine_Width = 0, $iWrap_Count, $iLast_Word, $sTest_Line
	; Declare and fill Size structure
	Local $tSize = DllStructCreate("int X;int Y")
	DllStructSetData($tSize, "X", 0)
	DllStructSetData($tSize, "Y", 0)

	; Ensure EoL is @CRLF and break text into lines
	$sText = StringRegExpReplace($sText, "((?<!\x0d)\x0a|\x0d(?!\x0a))", @CRLF)
	Local $asLines = StringSplit($sText, @CRLF, 1)

	; For each line
	For $i = 1 To $asLines[0]
		; Expand tabs if required
		If $iExpTab Then
			$asLines[$i] = StringReplace($asLines[$i], @TAB, " XXXXXXXX")
		EndIf
		; Size line
		$iLine_Length = StringLen($asLines[$i])
		DllCall("gdi32.dll", "bool", "GetTextExtentPoint32W", "handle", $hDC, "wstr", $asLines[$i], "int", $iLine_Length, "ptr", DllStructGetPtr($tSize))
		If @error Then Return SetError(2, _StringSize_Error_Close(6, $hDC, $hFont, $hLabel), 0)
		If DllStructGetData($tSize, "X") > $iLine_Width Then $iLine_Width = DllStructGetData($tSize, "X")
		If DllStructGetData($tSize, "Y") > $iLine_Height Then $iLine_Height = DllStructGetData($tSize, "Y")
	Next

	; Check if $iMaxWidth has been both set and exceeded
	If $iMaxWidth <> 0 And $iLine_Width > $iMaxWidth Then ; Wrapping required
		; For each Line
		For $j = 1 To $asLines[0]
			; Size line unwrapped
			$iLine_Length = StringLen($asLines[$j])
			DllCall("gdi32.dll", "bool", "GetTextExtentPoint32W", "handle", $hDC, "wstr", $asLines[$j], "int", $iLine_Length, "ptr", DllStructGetPtr($tSize))
			If @error Then Return SetError(2, _StringSize_Error_Close(6, $hDC, $hFont, $hLabel), 0)
			; Check wrap status
			If DllStructGetData($tSize, "X") < $iMaxWidth - 4 Then
				; No wrap needed so count line and store
				$iLine_Count += 1
				$avSize_Info[0] &= $asLines[$j] & @CRLF
			Else
				; Wrap needed so zero counter for wrapped lines
				$iWrap_Count = 0
				; Build line to max width
				While 1
					; Zero line width
					$iLine_Width = 0
					; Initialise pointer for end of word
					$iLast_Word = 0
					; Add characters until EOL or maximum width reached
					For $i = 1 To StringLen($asLines[$j])
						; Is this just past a word ending?
						If StringMid($asLines[$j], $i, 1) = " " Then $iLast_Word = $i - 1
						; Increase line by one character
						$sTest_Line = StringMid($asLines[$j], 1, $i)
						; Get line length
						$iLine_Length = StringLen($sTest_Line)
						DllCall("gdi32.dll", "bool", "GetTextExtentPoint32W", "handle", $hDC, "wstr", $sTest_Line, "int", $iLine_Length, "ptr", DllStructGetPtr($tSize))
						If @error Then Return SetError(2, _StringSize_Error_Close(6, $hDC, $hFont, $hLabel), 0)
						$iLine_Width = DllStructGetData($tSize, "X")
						; If too long exit the loop
						If $iLine_Width >= $iMaxWidth - 4 Then ExitLoop
					Next
					; End of the line of text?
					If $i > StringLen($asLines[$j]) Then
						; Yes, so add final line to count
						$iWrap_Count += 1
						; Store line
						$avSize_Info[0] &= $sTest_Line & @CRLF
						ExitLoop
					Else
						; No, but add line just completed to count
						$iWrap_Count += 1
						; Check at least 1 word completed or return error
						If $iLast_Word = 0 Then Return SetError(3, _StringSize_Error_Close(0, $hDC, $hFont, $hLabel), 0)
						; Store line up to end of last word
						$avSize_Info[0] &= StringLeft($sTest_Line, $iLast_Word) & @CRLF
						; Strip string to point reached
						$asLines[$j] = StringTrimLeft($asLines[$j], $iLast_Word)
						; Trim leading whitespace
						$asLines[$j] = StringStripWS($asLines[$j], 1)
						; Repeat with remaining characters in line
					EndIf
				WEnd
				; Add the number of wrapped lines to the count
				$iLine_Count += $iWrap_Count
			EndIf
		Next
		; Reset any tab expansions
		If $iExpTab Then
			$avSize_Info[0] = StringRegExpReplace($avSize_Info[0], "\x20?XXXXXXXX", @TAB)
		EndIf
		; Complete return array
		$avSize_Info[1] = $iLine_Height
		$avSize_Info[2] = $iMaxWidth
		; Convert lines to pixels and add drop margin
		$avSize_Info[3] = ($iLine_Count * $iLine_Height) + 4
	Else ; No wrapping required
		; Create return array (add drop margin to height)
		Local $avSize_Info[4] = [$sText, $iLine_Height, $iLine_Width, ($asLines[0] * $iLine_Height) + 4]
	EndIf

	; Clear up
	DllCall("gdi32.dll", "handle", "SelectObject", "handle", $hDC, "handle", $hPrevFont)
	DllCall("gdi32.dll", "bool", "DeleteObject", "handle", $hFont)
	DllCall("user32.dll", "int", "ReleaseDC", "hwnd", 0, "handle", $hDC)
	If $hLabel Then GUICtrlDelete($hLabel)

	Return $avSize_Info

EndFunc   ;==>_StringSize

; #INTERNAL_USE_ONLY#============================================================================================================
; Name...........: _StringSize_Error_Close
; Description ...: Releases DC and deleted font object if required after error
; Syntax ........: _StringSize_Error_Close ($iExtCode, $hDC, $hGUI)
; Parameters ....: $iExtCode   - code to return
;                  $hDC, $hGUI - handles as set in _StringSize function
; Return value ..: $iExtCode as passed
; Author ........: Melba23
; Modified.......:
; Remarks .......: This function is used internally by _StringSize
; ===============================================================================================================================
Func _StringSize_Error_Close($iExtCode, $hDC = 0, $hFont = 0, $hLabel = 0)

	If $hFont <> 0 Then DllCall("gdi32.dll", "bool", "DeleteObject", "handle", $hFont)
	If $hDC <> 0 Then DllCall("user32.dll", "int", "ReleaseDC", "hwnd", 0, "handle", $hDC)
	If $hLabel Then GUICtrlDelete($hLabel)

	Return $iExtCode

EndFunc   ;==>_StringSize_Error_Close

; #INTERNAL_USE_ONLY#============================================================================================================
; Name...........: _StringSize_DefaultFontName
; Description ...: Determines Windows default font
; Syntax ........: _StringSize_DefaultFontName()
; Parameters ....: None
; Return values .: Success - Returns name of system default font
;                  Failure - Returns "Tahoma"
; Author ........: Melba23, based on some original code by Larrydalooza
; Modified.......:
; Remarks .......: This function is used internally by _StringSize
; ===============================================================================================================================
Func _StringSize_DefaultFontName()

	; Get default system font data
	Local $tNONCLIENTMETRICS = DllStructCreate("uint;int;int;int;int;int;byte[60];int;int;byte[60];int;int;byte[60];byte[60];byte[60]")
	DllStructSetData($tNONCLIENTMETRICS, 1, DllStructGetSize($tNONCLIENTMETRICS))
	DllCall("user32.dll", "int", "SystemParametersInfo", "int", 41, "int", DllStructGetSize($tNONCLIENTMETRICS), "ptr", DllStructGetPtr($tNONCLIENTMETRICS), "int", 0)
	Local $tLOGFONT = DllStructCreate("long;long;long;long;long;byte;byte;byte;byte;byte;byte;byte;byte;char[32]", DllStructGetPtr($tNONCLIENTMETRICS, 13))
	If IsString(DllStructGetData($tLOGFONT, 14)) Then
		Return DllStructGetData($tLOGFONT, 14)
	Else
		Return "Tahoma"
	EndIf

EndFunc   ;==>_StringSize_DefaultFontName
